@@ -22,7 +22,7 @@ from sqlalchemy.orm.query import Query
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import is_
-from sqlalchemy.testing import is_not_
+from sqlalchemy.testing import is_not
 from sqlalchemy.testing import mock
 from sqlalchemy.testing.assertsql import CompiledSQL
 from test.orm import _fixtures
@@ -108,7 +108,7 @@ class StateChangeTest(BakedTest):
         q1 = self.bakery(l1)
 
         q2 = q1.with_criteria(l2)
-        is_not_(q2, q1)
+        is_not(q2, q1)
 
         self._assert_cache_key(q1._cache_key, [l1])
         self._assert_cache_key(q2._cache_key, [l1, l2])
@@ -126,7 +126,7 @@ class StateChangeTest(BakedTest):
         q1 = self.bakery(l1)
 
         q2 = q1 + l2
-        is_not_(q2, q1)
+        is_not(q2, q1)
 
         self._assert_cache_key(q1._cache_key, [l1])
         self._assert_cache_key(q2._cache_key, [l1, l2])
@@ -1112,7 +1112,7 @@ class LazyLoaderTest(testing.AssertsCompiledSQL, BakedTest):
                 ad.dingalings
         l2 = len(lru)
         eq_(l1, 0)
-        eq_(l2, 1)
+        eq_(l2, 0)
 
     def test_unsafe_bound_option_cancels_bake(self):
         User, Address, Dingaling = self._o2m_twolevel_fixture(lazy="joined")
@@ -1143,7 +1143,7 @@ class LazyLoaderTest(testing.AssertsCompiledSQL, BakedTest):
                 ad.dingalings
         l2 = len(lru)
         eq_(l1, 0)
-        eq_(l2, 1)
+        eq_(l2, 0)
 
     def test_safe_unbound_option_allows_bake(self):
         User, Address, Dingaling = self._o2m_twolevel_fixture(lazy="joined")
