@@ -130,6 +130,50 @@ class O2MTest(fixtures.MappedTest):
         eq_(result[1].parent_foo.data, "foo #1")
 
 
+class ColExpressionsTest(fixtures.DeclarativeMappedTest):
+    __backend__ = True
+
+    @classmethod
+    def setup_classes(cls):
+        Base = cls.DeclarativeBasic
+
+        class A(Base):
+            __tablename__ = "a"
+            id = Column(
+                Integer, primary_key=True, test_needs_autoincrement=True
+            )
+            type = Column(String(10))
+            __mapper_args__ = {
+                "polymorphic_on": type,
+                "polymorphic_identity": "a",
+            }
+
+        class B(A):
+            __tablename__ = "b"
+            id = Column(ForeignKey("a.id"), primary_key=True)
+            data = Column(Integer)
+            __mapper_args__ = {"polymorphic_identity": "b"}
+
+    @classmethod
+    def insert_data(cls, connection):
+        A, B = cls.classes("A", "B")
+        s = Session(connection)
+
+        s.add_all([B(data=5), B(data=7)])
+        s.commit()
+
+    def test_group_by(self):
+        B = self.classes.B
+        s = Session()
+
+        rows = (
+            s.query(B.id.expressions[0], B.id.expressions[1], func.sum(B.data))
+            .group_by(*B.id.expressions)
+            .all()
+        )
+        eq_(rows, [(1, 1, 5), (2, 2, 7)])
+
+
 class PolyExpressionEagerLoad(fixtures.DeclarativeMappedTest):
     run_setup_mappers = "once"
     __dialect__ = "default"
@@ -159,10 +203,10 @@ class PolyExpressionEagerLoad(fixtures.DeclarativeMappedTest):
             __mapper_args__ = {"polymorphic_identity": "b"}
 
     @classmethod
-    def insert_data(cls):
+    def insert_data(cls, connection):
         A = cls.classes.A
 
-        session = Session(testing.db)
+        session = Session(connection)
         session.add_all(
             [
                 A(id=1, discriminator="a"),
@@ -287,7 +331,7 @@ class PolymorphicResolutionMultiLevel(
 class PolymorphicOnNotLocalTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
-        t1 = Table(
+        Table(
             "t1",
             metadata,
             Column(
@@ -296,7 +340,7 @@ class PolymorphicOnNotLocalTest(fixtures.MappedTest):
             Column("x", String(10)),
             Column("q", String(10)),
         )
-        t2 = Table(
+        Table(
             "t2",
             metadata,
             Column(
@@ -332,21 +376,18 @@ class PolymorphicOnNotLocalTest(fixtures.MappedTest):
         )
 
     def test_polymorphic_on_non_expr_prop(self):
-        t2, t1 = self.tables.t2, self.tables.t1
+        t2 = self.tables.t2
         Parent = self.classes.Parent
 
-        t1t2_join = select([t1.c.x], from_obj=[t1.join(t2)]).alias()
-
-        def go():
-            interface_m = mapper(
-                Parent, t2, polymorphic_on=lambda: "hi", polymorphic_identity=0
-            )
-
         assert_raises_message(
             sa_exc.ArgumentError,
             "Only direct column-mapped property or "
             "SQL expression can be passed for polymorphic_on",
-            go,
+            mapper,
+            Parent,
+            t2,
+            polymorphic_on=lambda: "hi",
+            polymorphic_identity=0,
         )
 
     def test_polymorphic_on_not_present_col(self):
@@ -356,7 +397,7 @@ class PolymorphicOnNotLocalTest(fixtures.MappedTest):
 
         def go():
             t1t2_join_2 = select([t1.c.q], from_obj=[t1.join(t2)]).alias()
-            interface_m = mapper(
+            mapper(
                 Parent,
                 t2,
                 polymorphic_on=t1t2_join.c.x,
@@ -384,7 +425,7 @@ class PolymorphicOnNotLocalTest(fixtures.MappedTest):
             polymorphic_identity=0,
         )
 
-    def test_polymorpic_on_not_in_with_poly(self):
+    def test_polymorphic_on_not_in_with_poly(self):
         t2, t1 = self.tables.t2, self.tables.t1
         Parent = self.classes.Parent
 
@@ -393,7 +434,7 @@ class PolymorphicOnNotLocalTest(fixtures.MappedTest):
         # if with_polymorphic, but its not present, not OK
         def go():
             t1t2_join_2 = select([t1.c.q], from_obj=[t1.join(t2)]).alias()
-            interface_m = mapper(
+            mapper(
                 Parent,
                 t2,
                 polymorphic_on=t1t2_join.c.x,
@@ -447,7 +488,7 @@ class PolymorphicOnNotLocalTest(fixtures.MappedTest):
     def test_polymorphic_on_expr_implicit_map_no_label_single(self):
         """test that single_table_criterion is propagated
         with a standalone expr"""
-        t2, t1 = self.tables.t2, self.tables.t1
+        t1 = self.tables.t1
         Parent, Child = self.classes.Parent, self.classes.Child
         expr = case([(t1.c.x == "p", "parent"), (t1.c.x == "c", "child")])
         mapper(Parent, t1, polymorphic_identity="parent", polymorphic_on=expr)
@@ -458,7 +499,7 @@ class PolymorphicOnNotLocalTest(fixtures.MappedTest):
     def test_polymorphic_on_expr_implicit_map_w_label_single(self):
         """test that single_table_criterion is propagated
         with a standalone expr"""
-        t2, t1 = self.tables.t2, self.tables.t1
+        t1 = self.tables.t1
         Parent, Child = self.classes.Parent, self.classes.Child
         expr = case(
             [(t1.c.x == "p", "parent"), (t1.c.x == "c", "child")]
@@ -501,8 +542,8 @@ class PolymorphicOnNotLocalTest(fixtures.MappedTest):
         self._roundtrip(parent_ident="p", child_ident="c")
 
     def test_polymorphic_on_synonym(self):
-        t2, t1 = self.tables.t2, self.tables.t1
-        Parent, Child = self.classes.Parent, self.classes.Child
+        t1 = self.tables.t1
+        Parent = self.classes.Parent
         cprop = column_property(t1.c.x)
         assert_raises_message(
             sa_exc.ArgumentError,
@@ -852,8 +893,7 @@ class PolymorphicAttributeManagementTest(fixtures.MappedTest):
         )
 
     def test_entirely_oob_assignment(self):
-        """test warn on an unknown polymorphic identity.
-        """
+        """test warn on an unknown polymorphic identity."""
         B = self.classes.B
 
         sess = Session()
@@ -1392,7 +1432,7 @@ class FlushTest(fixtures.MappedTest):
         class Admin(User):
             pass
 
-        role_mapper = mapper(Role, roles)
+        mapper(Role, roles)
         user_mapper = mapper(
             User,
             users,
@@ -1402,7 +1442,7 @@ class FlushTest(fixtures.MappedTest):
                 )
             },
         )
-        admin_mapper = mapper(Admin, admins, inherits=user_mapper)
+        mapper(Admin, admins, inherits=user_mapper)
         sess = create_session()
         adminrole = Role()
         sess.add(adminrole)
@@ -1440,7 +1480,7 @@ class FlushTest(fixtures.MappedTest):
         class Admin(User):
             pass
 
-        role_mapper = mapper(Role, roles)
+        mapper(Role, roles)
         user_mapper = mapper(
             User,
             users,
@@ -1451,7 +1491,7 @@ class FlushTest(fixtures.MappedTest):
             },
         )
 
-        admin_mapper = mapper(Admin, admins, inherits=user_mapper)
+        mapper(Admin, admins, inherits=user_mapper)
 
         # create roles
         adminrole = Role("admin")
@@ -1876,7 +1916,7 @@ class VersioningTest(fixtures.MappedTest):
 
         assert_raises(
             orm_exc.StaleDataError,
-            sess2.query(Base).with_lockmode("read").get,
+            sess2.query(Base).with_for_update(read=True).get,
             s1.id,
         )
 
@@ -1975,14 +2015,18 @@ class DistinctPKTest(fixtures.MappedTest):
             pass
 
     @classmethod
-    def insert_data(cls):
+    def insert_data(cls, connection):
         person_insert = person_table.insert()
-        person_insert.execute(id=1, name="alice")
-        person_insert.execute(id=2, name="bob")
+        connection.execute(person_insert, dict(id=1, name="alice"))
+        connection.execute(person_insert, dict(id=2, name="bob"))
 
         employee_insert = employee_table.insert()
-        employee_insert.execute(id=2, salary=250, person_id=1)  # alice
-        employee_insert.execute(id=3, salary=200, person_id=2)  # bob
+        connection.execute(
+            employee_insert, dict(id=2, salary=250, person_id=1)
+        )  # alice
+        connection.execute(
+            employee_insert, dict(id=3, salary=200, person_id=2)
+        )  # bob
 
     def test_implicit(self):
         person_mapper = mapper(Person, person_table)
@@ -2010,7 +2054,7 @@ class DistinctPKTest(fixtures.MappedTest):
         )
         assert_raises_message(
             sa_exc.SAWarning,
-            r"On mapper Mapper\|Employee\|employees, "
+            r"On mapper mapped class Employee->employees, "
             "primary key column 'persons.id' is being "
             "combined with distinct primary key column 'employees.eid' "
             "in attribute 'id'. Use explicit properties to give "
@@ -2300,7 +2344,7 @@ class OverrideColKeyTest(fixtures.MappedTest):
         )
 
     def test_plain_descriptor(self):
-        """test that descriptors prevent inheritance from propigating
+        """test that descriptors prevent inheritance from propagating
         properties to subclasses."""
 
         class Base(object):
@@ -2321,7 +2365,7 @@ class OverrideColKeyTest(fixtures.MappedTest):
         assert sess.query(Sub).one().data == "im the data"
 
     def test_custom_descriptor(self):
-        """test that descriptors prevent inheritance from propigating
+        """test that descriptors prevent inheritance from propagating
         properties to subclasses."""
 
         class MyDesc(object):
@@ -2488,7 +2532,7 @@ class OptimizedLoadTest(fixtures.MappedTest):
         )
 
     def test_optimized_passes(self):
-        """"test that the 'optimized load' routine doesn't crash when
+        """ "test that the 'optimized load' routine doesn't crash when
         a column in the join condition is not available."""
 
         base, sub = self.tables.base, self.tables.sub
@@ -2874,9 +2918,7 @@ class InhCondTest(fixtures.TestBase):
             Column("id", Integer, ForeignKey("base.id"), primary_key=True),
             Column("order_id", Integer, ForeignKey("order.foo")),
         )
-        order_table = Table(
-            "order", m, Column("id", Integer, primary_key=True)
-        )
+        Table("order", m, Column("id", Integer, primary_key=True))
 
         class Base(object):
             pass
@@ -2983,7 +3025,7 @@ class InhCondTest(fixtures.TestBase):
 class PKDiscriminatorTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
-        parents = Table(
+        Table(
             "parents",
             metadata,
             Column(
@@ -2992,7 +3034,7 @@ class PKDiscriminatorTest(fixtures.MappedTest):
             Column("name", String(60)),
         )
 
-        children = Table(
+        Table(
             "children",
             metadata,
             Column("id", Integer, ForeignKey("parents.id"), primary_key=True),
@@ -3266,7 +3308,7 @@ class PolymorphicUnionTest(fixtures.TestBase, testing.AssertsCompiledSQL):
 class NameConflictTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
-        content = Table(
+        Table(
             "content",
             metadata,
             Column(
@@ -3274,7 +3316,7 @@ class NameConflictTest(fixtures.MappedTest):
             ),
             Column("type", String(30)),
         )
-        foo = Table(
+        Table(
             "foo",
             metadata,
             Column("id", Integer, ForeignKey("content.id"), primary_key=True),
