@@ -1,9 +1,11 @@
+from sqlalchemy import cast
 from sqlalchemy import event
 from sqlalchemy import exc
 from sqlalchemy import FetchedValue
 from sqlalchemy import ForeignKey
 from sqlalchemy import func
 from sqlalchemy import Integer
+from sqlalchemy import JSON
 from sqlalchemy import literal
 from sqlalchemy import select
 from sqlalchemy import String
@@ -476,6 +478,74 @@ class RudimentaryFlushTest(UOWTest):
             ),
         )
 
+    def test_many_to_one_del_attr(self):
+        users, Address, addresses, User = (
+            self.tables.users,
+            self.classes.Address,
+            self.tables.addresses,
+            self.classes.User,
+        )
+
+        mapper(User, users)
+        mapper(Address, addresses, properties={"user": relationship(User)})
+        sess = create_session()
+
+        u1 = User(name="u1")
+        a1, a2 = (
+            Address(email_address="a1", user=u1),
+            Address(email_address="a2", user=u1),
+        )
+        sess.add_all([a1, a2])
+        sess.flush()
+
+        del a1.user
+        self.assert_sql_execution(
+            testing.db,
+            sess.flush,
+            CompiledSQL(
+                "UPDATE addresses SET user_id=:user_id WHERE "
+                "addresses.id = :addresses_id",
+                lambda ctx: [{"addresses_id": a1.id, "user_id": None}],
+            ),
+        )
+
+    def test_many_to_one_del_attr_unloaded(self):
+        users, Address, addresses, User = (
+            self.tables.users,
+            self.classes.Address,
+            self.tables.addresses,
+            self.classes.User,
+        )
+
+        mapper(User, users)
+        mapper(Address, addresses, properties={"user": relationship(User)})
+        sess = create_session()
+
+        u1 = User(name="u1")
+        a1, a2 = (
+            Address(email_address="a1", user=u1),
+            Address(email_address="a2", user=u1),
+        )
+        sess.add_all([a1, a2])
+        sess.flush()
+
+        # trying to guarantee that the history only includes
+        # PASSIVE_NO_RESULT for "deleted" and nothing else
+        sess.expunge(u1)
+        sess.expire(a1, ["user"])
+        del a1.user
+
+        sess.add(a1)
+        self.assert_sql_execution(
+            testing.db,
+            sess.flush,
+            CompiledSQL(
+                "UPDATE addresses SET user_id=:user_id WHERE "
+                "addresses.id = :addresses_id",
+                lambda ctx: [{"addresses_id": a1.id, "user_id": None}],
+            ),
+        )
+
     def test_natural_ordering(self):
         """test that unconnected items take relationship()
         into account regardless."""
@@ -1499,6 +1569,8 @@ class RowswitchM2OTest(fixtures.MappedTest):
 
 
 class BasicStaleChecksTest(fixtures.MappedTest):
+    __backend__ = True
+
     @classmethod
     def define_tables(cls, metadata):
         Table(
@@ -1688,6 +1760,93 @@ class BasicStaleChecksTest(fixtures.MappedTest):
             sess.flush,
         )
 
+    def test_update_single_broken_multi_rowcount_still_raises(self):
+        # raise occurs for single row UPDATE that misses even if
+        # supports_sane_multi_rowcount is False
+        Parent, Child = self._fixture()
+        sess = Session()
+        p1 = Parent(id=1, data=2, child=None)
+        sess.add(p1)
+        sess.flush()
+
+        sess.execute(self.tables.parent.delete())
+        p1.data = 3
+
+        with patch.object(
+            config.db.dialect, "supports_sane_multi_rowcount", False
+        ):
+            assert_raises_message(
+                orm_exc.StaleDataError,
+                r"UPDATE statement on table 'parent' expected to "
+                r"update 1 row\(s\); 0 were matched.",
+                sess.flush,
+            )
+
+    def test_update_multi_broken_multi_rowcount_doesnt_raise(self):
+        # raise does not occur for multirow UPDATE that misses if
+        # supports_sane_multi_rowcount is False, even if rowcount is still
+        # correct
+        Parent, Child = self._fixture()
+        sess = Session()
+        p1 = Parent(id=1, data=2, child=None)
+        p2 = Parent(id=2, data=3, child=None)
+        sess.add_all([p1, p2])
+        sess.flush()
+
+        sess.execute(self.tables.parent.delete())
+        p1.data = 3
+        p2.data = 4
+
+        with patch.object(
+            config.db.dialect, "supports_sane_multi_rowcount", False
+        ):
+            # no raise
+            sess.flush()
+
+    def test_delete_single_broken_multi_rowcount_still_warns(self):
+        Parent, Child = self._fixture()
+        sess = Session()
+        p1 = Parent(id=1, data=2, child=None)
+        sess.add(p1)
+        sess.flush()
+        sess.flush()
+
+        sess.execute(self.tables.parent.delete())
+        sess.delete(p1)
+
+        # only one row, so it warns
+        with patch.object(
+            config.db.dialect, "supports_sane_multi_rowcount", False
+        ):
+            assert_raises_message(
+                exc.SAWarning,
+                r"DELETE statement on table 'parent' expected to "
+                r"delete 1 row\(s\); 0 were matched.",
+                sess.flush,
+            )
+
+    def test_delete_multi_broken_multi_rowcount_doesnt_warn(self):
+        Parent, Child = self._fixture()
+        sess = Session()
+        p1 = Parent(id=1, data=2, child=None)
+        p2 = Parent(id=2, data=3, child=None)
+        sess.add_all([p1, p2])
+        sess.flush()
+
+        sess.execute(self.tables.parent.delete())
+        sess.delete(p1)
+        sess.delete(p2)
+
+        # if the dialect reports supports_sane_multi_rowcount as false,
+        # if there were more than one row deleted, need to ensure the
+        # rowcount result is ignored.  psycopg2 + batch mode reports the
+        # wrong number, not -1. see issue #4661
+        with patch.object(
+            config.db.dialect, "supports_sane_multi_rowcount", False
+        ):
+            # no warning
+            sess.flush()
+
     def test_delete_multi_missing_allow(self):
         Parent, Child = self._fixture(confirm_deleted_rows=False)
         sess = Session()
@@ -2601,6 +2760,20 @@ class NullEvaluatingTest(fixtures.MappedTest, testing.AssertsExecutionResults):
             ),
         )
 
+        if testing.requires.json_type.enabled:
+            Table(
+                "test_has_json",
+                metadata,
+                Column(
+                    "id",
+                    Integer,
+                    primary_key=True,
+                    test_needs_autoincrement=True,
+                ),
+                Column("data", JSON(none_as_null=True).evaluates_none()),
+                Column("data_null", JSON(none_as_null=True)),
+            )
+
     @classmethod
     def setup_classes(cls):
         class Thing(cls.Basic):
@@ -2609,6 +2782,9 @@ class NullEvaluatingTest(fixtures.MappedTest, testing.AssertsExecutionResults):
         class AltNameThing(cls.Basic):
             pass
 
+        class JSONThing(cls.Basic):
+            pass
+
     @classmethod
     def setup_mappers(cls):
         Thing = cls.classes.Thing
@@ -2618,6 +2794,9 @@ class NullEvaluatingTest(fixtures.MappedTest, testing.AssertsExecutionResults):
 
         mapper(AltNameThing, cls.tables.test_w_renames, column_prefix="_foo_")
 
+        if testing.requires.json_type.enabled:
+            mapper(cls.classes.JSONThing, cls.tables.test_has_json)
+
     def _assert_col(self, name, value):
         Thing, AltNameThing = self.classes.Thing, self.classes.AltNameThing
         s = Session()
@@ -2751,3 +2930,14 @@ class NullEvaluatingTest(fixtures.MappedTest, testing.AssertsExecutionResults):
         self._test_bulk_insert_novalue(
             "builtin_evals_null_default", "default_val"
         )
+
+    @testing.requires.json_type
+    def test_json_none_as_null(self):
+        JSONThing = self.classes.JSONThing
+
+        s = Session()
+        f1 = JSONThing(data=None, data_null=None)
+        s.add(f1)
+        s.commit()
+        eq_(s.query(cast(JSONThing.data, String)).scalar(), "null")
+        eq_(s.query(cast(JSONThing.data_null, String)).scalar(), None)
